job "myheats-demo" {
  datacenters = ["dc1"]

  vault {}

  priority = 80

  group "server" {
    count = 1

    # /alloc/data
    ephemeral_disk {
      size    = 500
    }

    volume "tls" {
      type = "csi"
      source = "certbot"
      access_mode = "multi-node-multi-writer"
      attachment_mode = "file-system"
    }

    network {
      port "frontend_http" {
        # Nginx default image always listens on 80
        to = 80
      }
      port "api_http" {}
      port "proxy_https" {
        static = 44396
      }
    }

    task "nginx_proxy" {
      driver = "podman"

      config {
        image = "docker.io/library/nginx:stable-alpine"
        ports = ["proxy_https"]
        volumes = [
          # mount the templated config from the task directory to the container
          "local/nginx-proxy.conf:/etc/nginx/conf.d/default.conf",
        ]
      }

      volume_mount {
        volume = "tls"
        destination = "/etc/letsencrypt"
      }

      template {
        destination = "${NOMAD_TASK_DIR}/nginx-proxy.conf"
        data = file("./templates/nginx-proxy.conf.tmpl")
      }

      resources {
        memory = 50
        memory_max = 256
        cpu    = 200
      }
    }

    # Prepare latest MyHeats frontend release
    task "build_frontend" {
      driver = "podman"
 
      template {
        # render sensitive env vars in a template from Vault secrets
        env = true
        destination = "${NOMAD_SECRETS_DIR}/frontend-env"
        data = file("./templates/frontend-env.local.tmpl")
      }

      artifact {
        source = "https://code.in0rdr.ch/myheats/archive/myheats-master.tar.gz"
        destination = "local/myheats.tar.gz"
        options {
          archive = false
          filename = "myheats.tar.gz"
        }
        mode = "file"
      }

      config {
        image = "docker.io/node:18-alpine"
        command = "/bin/sh"
        args = [ "-c", <<EOT
          mkdir -p /alloc/data/myheats && rm -rf /alloc/data/myheats/*
          tar --strip-components=1 -C /alloc/data/myheats -xvf /local/myheats.tar.gz
          cd /alloc/data/myheats
          npm ci
          npm run build
        EOT
        ]
        volumes = [
          # mount the templated config from the task directory to the container
          "secrets/frontend-env:/${NOMAD_TASK_DIR}/local/app/.env.local",
        ]
      }
 
      resources {
        memory = 256
        memory_max = 512
        cpu = 300
      }
 
      lifecycle {
        hook = "prestart"
        sidecar = false
      }
    }

    task "frontend" {
      driver = "podman"

      template {
        destination = "${NOMAD_TASK_DIR}/nginx-frontend.conf"
        data = file("./templates/nginx-frontend.conf.tmpl")
      }

      config {
        image = "docker.io/nginx:alpine"
        ports = ["frontend_http"]
        command = "/bin/sh"
        args = ["-c", <<EOT
          echo  "Copyng new distribution from workdir"
          rm -rf /usr/share/nginx/html/*
          cp -r /alloc/data/myheats/dist/* /usr/share/nginx/html/
          nginx -g 'daemon off;'
        EOT
        ]
        volumes = [
          "local/nginx-frontend.conf:/etc/nginx/conf.d/default.conf",
        ]
      }

      resources {
        memory = 100
        memory_max = 256
        cpu    = 300
      }
    }

    task "backend" {
      driver = "podman"

      config {
        image = "docker.io/node:18-alpine"
        command = "/bin/sh"
        args = ["-c", "cd /alloc/data/myheats && npm run api"]
        force_pull = true
        ports = ["api_http"]
        volumes = [
          # mount the templated config from the task directory to the container
          "secrets/backend-env:/app/.env.local",
        ]
      }

      template {
        # render sensitive env vars in a template from Vault secrets
        env = true
        destination = "${NOMAD_SECRETS_DIR}/backend-env"
        data = file("./templates/backend-env.local.tmpl")
      }

      resources {
        memory = 124
        memory_max = 512
        cpu    = 300
      }
    }
  }
}
