job "writefreely" {
  datacenters = ["dc1"]

  vault {}

  group "server" {
    count = 1

    volume "tls" {
      type = "csi"
      source = "certbot"
      access_mode = "multi-node-multi-writer"
      attachment_mode = "file-system"
    }
    volume "writefreely" {
      type = "csi"
      source = "writefreely"
      access_mode = "multi-node-multi-writer"
      attachment_mode = "file-system"
    }

    network {
      port "web" {}
      port "https" {
        static = 44394
      }
    }

    # Prepare database schema
    # https://github.com/writefreely/writefreely/blob/develop/docker-setup.sh
    task "db-upgrade" {
      driver = "podman"

      config {
        image = "docker.io/arm64v8/mysql:latest"
        command = "/bin/sh"
        args = ["-c", "mysql < $NOMAD_TASK_DIR/schema.sql"]
        volumes = [
          # mount the templated config from the task directory to the container
          "local/my.cnf:/etc/mysql/conf.d/my.cnf",
        ]
        
      }

      template {
        # Fetch sensitive db connection vars from Vault secrets
        destination = "${NOMAD_TASK_DIR}/my.cnf"
        data = file("./templates/my.cnf.tmpl")
      }
      template {
        # Create db tables
        destination = "${NOMAD_TASK_DIR}/schema.sql"
        data = file("./templates/schema.sql.tmpl")
      }

      resources {
        memory = 50
        memory_max = 128
        cpu = 100
      }

      lifecycle {
        hook = "prestart"
        sidecar = false
      }
    }

    # Generate keys used for the encryption of certain user data.
    # Because user data becomes unrecoverable without these keys,
    # this won't overwrite any existing key, and instead outputs a message.
    # https://github.com/writefreely/writefreely/blob/develop/docker-setup.sh
    task "gen-keys" {
      driver = "podman"

      config {
        image = "127.0.0.1:5000/writefreely:latest"
        entrypoint = [""]
        command = "/bin/sh"
        args = ["-c", "cmd/writefreely/writefreely --gen-keys || true"]
        volumes = [
          # mount the templated config from the task directory to the container
          "local/config.ini:/go/config.ini",
        ]
      }

      volume_mount {
        volume = "writefreely"
        destination = "/go/keys"
      }

      template {
        destination = "${NOMAD_TASK_DIR}/config.ini"
        data = file("./templates/config.ini.tmpl")
      }

      resources {
        memory = 50
        memory_max = 128
        cpu = 100
      }

      lifecycle {
        hook = "prestart"
        sidecar = false
      }
    }

    task "writefreely" {
      driver = "podman"

      config {
        image = "127.0.0.1:5000/writefreely:latest"
        force_pull = true
        ports = ["web"]
        volumes = [
          # mount the templated config from the task directory to the container
          "local/config.ini:/go/config.ini",
        ]
      }

      volume_mount {
        volume = "writefreely"
        destination = "/go/keys"
      }

      template {
        destination = "${NOMAD_TASK_DIR}/config.ini"
        data = file("./templates/config.ini.tmpl")
      }

      resources {
        memory = 50
        memory_max = 256
        cpu    = 200
      }
    }

    task "nginx" {
      driver = "podman"

      config {
        image = "docker.io/library/nginx:stable-alpine"
        ports = ["https"]
        volumes = [
          # mount the templated config from the task directory to the container
          "local/writefreely.conf:/etc/nginx/conf.d/writefreely.conf",
        ]
      }

      volume_mount {
        volume = "tls"
        destination = "/etc/letsencrypt"
      }

      template {
        destination = "${NOMAD_TASK_DIR}/writefreely.conf"
        data = file("./templates/nginx.conf.tmpl")
      }

      resources {
        memory = 50
        memory_max = 256
        cpu    = 200
      }
    }

  }
}
